package Datahub::Factory::Env;

use Datahub::Factory::Sane;

use Config::Simple;

use Moo;
use Catmandu;
use namespace::clean;

with 'Datahub::Factory::Logger';



1;

__END__

=head1 NAME

Datahub::Factory::Env - A Datahub::Factory configuration file loader

=head1 SYNOPSIS

=head1 DESCRIPTION

=cut

